/ The standard MicroCode from page 146 in Structured Computer Organization
/ by Andrew S. Tanenbaum
/ typed in by David M. O'Rourke & Steve LoBasso
/ Modifications made for compatibility with the MacSim Simulator
/ March 3, 1990

/ MicroCode below

start: mar := pc; rd;
       pc := pc+1; rd;
       ir := mbr; if n goto x28;
       tir := lshift(ir + ir); if n goto cjump;
       tir := lshift(tir); if n goto math;
       alu := tir; if n goto stod;

/  lodd instruction
       mar := ir; rd;
x7:    rd;
       ac := mbr; goto start;
stod:  mar := ir; mbr := ac; wr;          /  line 9: in book
x10:   wr; goto start;
math:  alu := tir; if n goto subd;        /  line 11: in book
       mar := ir; rd;
x13:   rd;
       ac := mbr + ac; goto start;

subd:  mar := ir; rd;                     /  line 15: in book
x16:   ac := ac + 1; rd;
       a := inv(mbr);
       ac := a + ac; goto start;

cjump: tir := lshift(tir); if n goto x25; /  line 19: in book
       alu := tir; if n goto jzer;
       alu := ac; if n goto start;
x22:   pc := band(ir,amask); goto start;
jzer:  alu := ac; if z goto x22;          /  line 23: in book
       goto start;

x25:   alu := tir; if n goto loco;
       pc := band(ir, amask); goto start;
loco:  ac := band(ir, amask); goto start; /  line 27: in book

x28:   tir := lshift(ir + ir); if n goto x40;
       tir := lshift(tir); if n goto mathl;
       alu := tir; if n goto stol;

/  lodl instruction
       a := ir + sp;
       mar := a; rd; goto x7;
stol:  a := ir + sp;                      /  line 33: in book
       mar := a; mbr := ac; wr; goto x10;
mathl: alu := tir; if n goto subl;        /  line 35: in book

/  addl instruction
       a := ir + sp;
       mar := a; rd; goto x13;

/  subl instruction
subl:  a := ir + sp;                      /  line 38: in book
       mar := a; rd; goto x16;
x40:   tir := lshift(tir); if n goto x46;
       alu := tir; if n goto jnze;

/  jneg instruction
       alu := ac; if n goto x22;
       goto start;

/  jnez instruction
jnze:  alu := ac; if z goto start;        /  line 44: in book
       pc := band(ir, amask); goto start;

x46:   tir := lshift(tir); if n goto x50;

/  call instruction
       sp := sp + -1;
       mar := sp; mbr := pc; wr;
       pc := band(ir, amask); wr; goto start;

x50:   tir := lshift(tir); if n goto x65;
       tir := lshift(tir); if n goto x59;
       alu := tir; if n goto popi;

/  pshi instruction
       mar := ac; rd;
       sp := sp + -1; rd;
       mar := sp; wr; goto x10;

/  popi instruction
popi:  mar := sp; sp := 1 + sp; rd;        /  line 56: in book
       rd;
       mar := ac; wr; goto x10;
x59:   alu := tir; if n goto pop;

/  push instruction
       sp := sp + -1;
       mar := sp; mbr := ac; wr; goto x10;
pop:   mar := sp; sp := 1 + sp; rd;      /  line 62: in book
       rd;
       ac := mbr; goto start;
x65:   tir := lshift(tir); if n goto x73;
       alu := tir; if n goto swap;

/  retn instruction
       mar := sp; sp := 1 + sp; rd;
       rd;
       pc := mbr; goto start;
swap:  a := ac;                          /  line 70: in book
       ac := sp;
       sp := a; goto start;
x73:   alu := tir; if n goto desp;
       a := band(ir, smask);
x75:   sp := sp + a; goto start;
    
desp:  a := band(ir, smask);             /  line 76: in book
       a := inv(a);
       a := a + 1; goto x75;

/
/ start symbol table for microcode
/
#
lodd,0x0000-12,
  indev=1,outdev=2,
  incntl=3, outcntl=4,
  inchar=5, inmod=6,
  outcmnd=7, outdata=8;
stod,0x1000-12,
  indev=1,outdev=2,
  incntl=3, outcntl=4,
  inchar=5, inmod=6,
  outcmnd=7, outdata=8;
addd,0x2000-12;
subd,0x3000-12;
jpos,0x4000-12;
jzer,0x5000-12;
jump,0x6000-12;
loco,0x7000-12;
lodl,0x8000-12;
stol,0x9000-12;
addl,0xa000-12;
subl,0xb000-12;
jneg,0xc000-12;
jnze,0xd000-12;
call,0xe000-12;
pshi,0xf000;
popi,0xf200;
push,0xf400;
pop, 0xf600;
retn,0xf800;
swap,0xfa00;
insp,0xfc00-8;
desp,0xfe00-8;
#